/*
 * Ben NanoNote LCD initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>
 * Copyright (C) 2001-2002 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "xburst_types.h"
#include "nanonote_gpm940b0.h"
#include "board-nanonote.h"

#define ALIGN(x,a)		__ALIGN_MASK((x),(typeof(x))(a)-1)
#define __ALIGN_MASK(x,mask)	(((x)+(mask))&~(mask))

unsigned long lcd_setmem(unsigned long addr)
{
        unsigned long size;

        size = lcd_get_size();

        /* Round up to nearest full page, or MMU section if defined */
        size = ALIGN(size, PAGE_SIZE);
        addr = ALIGN(addr - PAGE_SIZE + 1, PAGE_SIZE);

        /* Allocate pages for the frame buffer. */
        addr -= size;

        return addr;
}

#define N_BLK_VERT      2
#define N_BLK_HOR       3

static int test_colors[N_BLK_HOR * N_BLK_VERT] = {
	0x00ff0000,	0x0000ff00,	0x00ffff00,
	0x000000ff,	0x00ff00ff,	0x0000ffff,
};

static void test_pattern(void *lcd_base)
{
	unsigned short v_max  = panel_info.vl_row;
	unsigned short h_max  = panel_info.vl_col;
	unsigned short v_step = (v_max + N_BLK_VERT - 1) / N_BLK_VERT;
	unsigned short h_step = (h_max + N_BLK_HOR  - 1) / N_BLK_HOR;
	unsigned short v, h;
	unsigned char *pix = (unsigned char *)lcd_base;

	/* WARNING: Code silently assumes 8bit/pixel */
	for (v = 0; v < v_max; ++v) {
		unsigned char iy = v / v_step;
		for (h = 0; h < h_max; ++h) {
			unsigned char ix = N_BLK_HOR * iy + h / h_step;
			*pix++ = test_colors[ix];
		}
	}
}

void lcd_clear(void *lcd_base)
{
	test_pattern(lcd_base);
}

/* LCD initialisation. */

static void *lcd_base;

void lcd_init(void)
{
	__lcd_display_pin_init();
	__lcd_display_on();

	/* Start from the top of memory and obtain a framebuffer region. */

	lcd_base = (void *) lcd_setmem(get_memory_size());
	lcd_ctrl_init(lcd_base);
	lcd_clear(lcd_base);
	lcd_enable();
}
